@@ -49,10 +49,11 @@ namespace netxs::app::term
 {
     using events = netxs::events::userland::term;
 
-    const auto terminal_menu = [](bool full_size)
+    const auto terminal_menu = [](xml::settings& config)
     {
-        const static auto c3 = app::shared::c3;
-        const static auto x3 = app::shared::x3;
+        auto highlight_color = skin::color(tone::highlight);
+        auto c3 = highlight_color;
+        auto x3 = cell{ c3 }.alpha(0x00);
 
         auto items = app::shared::menu_list_type
         {
@@ -113,6 +114,9 @@ namespace netxs::app::term
                 };
                 boss.SUBMIT(tier::anycast, app::term::events::layout::wrapln, wrapln)
                 {
+                    auto highlight_color = skin::color(tone::highlight);
+                    auto c3 = highlight_color;
+                    auto x3 = cell{ c3 }.alpha(0x00);
                     //todo unify, get boss base colors, don't use x3
                     boss.color(wrapln == wrap::on ? 0xFF00ff00 : x3.fgc(), x3.bgc());
                 };
@@ -127,6 +131,9 @@ namespace netxs::app::term
                 };
                 boss.SUBMIT(tier::anycast, app::term::events::selmod, selmod)
                 {
+                    auto highlight_color = skin::color(tone::highlight);
+                    auto c3 = highlight_color;
+                    auto x3 = cell{ c3 }.alpha(0x00);
                     //todo unify, get boss base colors, don't use x3, make it configurable
                     switch (selmod)
                     {
@@ -159,6 +166,10 @@ namespace netxs::app::term
                             if (boss.client) boss.client->SIGNAL(tier::release, e2::data::text, "HTML-code");
                             boss.color(0xFFffff00, x3.bgc());
                             break;
+                        case clip::safetext:
+                            if (boss.client) boss.client->SIGNAL(tier::release, e2::data::text, "Protected");
+                            boss.color(0xFFffff00, x3.bgc());
+                            break;
                     }
                     boss.deface();
                 };
@@ -175,6 +186,9 @@ namespace netxs::app::term
                 };
                 boss.SUBMIT(tier::anycast, app::term::events::search::status, mode)
                 {
+                    auto highlight_color = skin::color(tone::highlight);
+                    auto c3 = highlight_color;
+                    auto x3 = cell{ c3 }.alpha(0x00);
                     //todo unify, get boss base colors, don't use x3
                     boss.color(mode & 2 ? 0xFF00ff00 : x3.fgc(), x3.bgc());
                 };
@@ -191,6 +205,9 @@ namespace netxs::app::term
                 };
                 boss.SUBMIT(tier::anycast, app::term::events::search::status, mode)
                 {
+                    auto highlight_color = skin::color(tone::highlight);
+                    auto c3 = highlight_color;
+                    auto x3 = cell{ c3 }.alpha(0x00);
                     //todo unify, get boss base colors, don't use x3
                     boss.color(mode & 1 ? 0xFF00ff00 : x3.fgc(), x3.bgc());
                 };
@@ -218,13 +235,17 @@ namespace netxs::app::term
                 };
             }},
         };
-        return app::shared::custom_menu(full_size, items);
+        config.cd("/config/term/", "/config/defapp/");
+        return app::shared::custom_menu(config, items);
     };
 
     namespace
     {
-        auto build = [](text cwd, text arg)
+        auto build = [](text cwd, text arg, xml::settings& config)
         {
+            auto menu_white = skin::color(tone::menu_white);
+            auto cB = menu_white;
+
             auto window = ui::cake::ctor();
             auto arg_shadow = view{ arg };
             auto term_type = shared::app_class(arg_shadow);
@@ -234,13 +255,11 @@ namespace netxs::app::term
                                                              ->plugin<pro::acryl>()
                                                              ->plugin<pro::cache>();
             auto object = window->attach(ui::fork::ctor(axis::Y))
-                                ->colors(whitelt, app::shared::term_menu_bg);
-                auto menu = object->attach(slot::_1, terminal_menu(true));
-                auto term_stat_area = object->attach(slot::_2, ui::fork::ctor(axis::Y));
+                                ->colors(cB.fgc(), cB.bgc());
+            auto term_stat_area = object->attach(slot::_2, ui::fork::ctor(axis::Y));
                     auto layers = term_stat_area->attach(slot::_1, ui::cake::ctor())
                                                 ->plugin<pro::limit>(dot_11, twod{ 400,200 });
                         auto scroll = layers->attach(ui::rail::ctor());
-                        {
                             auto min_size = twod{ 12,1 }; // mc crashes when window is too small
                             auto max_size = -dot_11;
                             auto forced_clamp = faux;
@@ -275,62 +294,81 @@ namespace netxs::app::term
                                     };
                                   });
 
-                            auto shell = os::get_shell();
-                            auto inst = scroll->attach(ui::term::ctor(cwd, arg.empty() ? shell + " -i"
-                                                                                       : arg));
+                            auto shell = os::get_shell() + " -i";
+                            auto inst = scroll->attach(ui::term::ctor(cwd, arg.empty() ? shell : arg, config));
 
-                            inst->attach_property(ui::term::events::colors::bg,      app::term::events::colors::bg)
-                                ->attach_property(ui::term::events::colors::fg,      app::term::events::colors::fg)
-                                ->attach_property(ui::term::events::selmod,          app::term::events::selmod)
-                                ->attach_property(ui::term::events::layout::wrapln,  app::term::events::layout::wrapln)
-                                ->attach_property(ui::term::events::layout::align,   app::term::events::layout::align)
-                                ->attach_property(ui::term::events::search::status,  app::term::events::search::status)
-                                ->invoke([](auto& boss)
-                                {
-                                    boss.SUBMIT(tier::anycast, app::term::events::cmd, cmd)
-                                    {
-                                        boss.exec_cmd(static_cast<ui::term::commands::ui::commands>(cmd));
-                                    };
-                                    boss.SUBMIT(tier::anycast, app::term::events::data::in, data)
-                                    {
-                                        boss.data_in(data);
-                                    };
-                                    boss.SUBMIT(tier::anycast, app::term::events::data::out, data)
-                                    {
-                                        boss.data_out(data);
-                                    };
-                                    //todo add color picker to the menu
-                                    boss.SUBMIT(tier::anycast, app::term::events::colors::bg, bg)
-                                    {
-                                        boss.set_bg_color(bg);
-                                    };
-                                    boss.SUBMIT(tier::anycast, app::term::events::colors::fg, fg)
-                                    {
-                                        boss.set_fg_color(fg);
-                                    };
-                                    boss.SUBMIT(tier::anycast, e2::form::prop::colors::any, clr)
-                                    {
-                                        auto deed = boss.bell::template protos<tier::anycast>();
-                                             if (deed == e2::form::prop::colors::bg.id) boss.SIGNAL(tier::anycast, app::term::events::colors::bg, clr);
-                                        else if (deed == e2::form::prop::colors::fg.id) boss.SIGNAL(tier::anycast, app::term::events::colors::fg, clr);
-                                    };
-                                    boss.SUBMIT(tier::anycast, e2::form::upon::started, root)
-                                    {
-                                        boss.start();
-                                    };
-                                    boss.SUBMIT(tier::anycast, app::term::events::search::forward, gear)
-                                    {
-                                        boss.search(gear, feed::fwd);
-                                    };
-                                    boss.SUBMIT(tier::anycast, app::term::events::search::reverse, gear)
-                                    {
-                                        boss.search(gear, feed::rev);
-                                    };
-                                });
-                        }
-                    auto scroll_bars = layers->attach(ui::fork::ctor());
-                        auto vt = scroll_bars->attach(slot::_2, ui::grip<axis::Y>::ctor(scroll));
-                        auto hz = term_stat_area->attach(slot::_2, ui::grip<axis::X>::ctor(scroll));
+                            auto scroll_bars = layers->attach(ui::fork::ctor());
+                                auto vt = scroll_bars->attach(slot::_2, ui::grip<axis::Y>::ctor(scroll));
+                                auto hz = term_stat_area->attach(slot::_2, ui::grip_fx2<axis::X>::ctor(scroll))
+                                                        ->plugin<pro::limit>(twod{ -1,1 }, twod{ -1,1 })
+                                                        ->invoke([&](auto& boss)
+                                                        {
+                                                            boss.SUBMIT(tier::anycast, app::term::events::colors::bg, bg)
+                                                            {
+                                                                boss.color(boss.color().bgc(bg).txt(""));
+                                                            };
+                                                        });
+
+            auto [slot1, cover, menu_data] = terminal_menu(config);
+            auto menu = object->attach(slot::_1, slot1);
+            auto menu_id = slot1->id;
+            cover->invoke([&](auto& boss)
+            {
+                boss.colors(cell{ cB }.inv(true).txt("▀"sv).link(menu_id));
+                boss.SUBMIT(tier::anycast, app::term::events::colors::bg, bg)
+                {
+                    boss.color(boss.color().fgc(bg));
+                };
+            });
+
+            inst->attach_property(ui::term::events::colors::bg,      app::term::events::colors::bg)
+                ->attach_property(ui::term::events::colors::fg,      app::term::events::colors::fg)
+                ->attach_property(ui::term::events::selmod,          app::term::events::selmod)
+                ->attach_property(ui::term::events::layout::wrapln,  app::term::events::layout::wrapln)
+                ->attach_property(ui::term::events::layout::align,   app::term::events::layout::align)
+                ->attach_property(ui::term::events::search::status,  app::term::events::search::status)
+                ->invoke([](auto& boss)
+                {
+                    boss.SUBMIT(tier::anycast, app::term::events::cmd, cmd)
+                    {
+                        boss.exec_cmd(static_cast<ui::term::commands::ui::commands>(cmd));
+                    };
+                    boss.SUBMIT(tier::anycast, app::term::events::data::in, data)
+                    {
+                        boss.data_in(data);
+                    };
+                    boss.SUBMIT(tier::anycast, app::term::events::data::out, data)
+                    {
+                        boss.data_out(data);
+                    };
+                    //todo add color picker to the menu
+                    boss.SUBMIT(tier::anycast, app::term::events::colors::bg, bg)
+                    {
+                        boss.set_bg_color(bg);
+                    };
+                    boss.SUBMIT(tier::anycast, app::term::events::colors::fg, fg)
+                    {
+                        boss.set_fg_color(fg);
+                    };
+                    boss.SUBMIT(tier::anycast, e2::form::prop::colors::any, clr)
+                    {
+                        auto deed = boss.bell::template protos<tier::anycast>();
+                             if (deed == e2::form::prop::colors::bg.id) boss.SIGNAL(tier::anycast, app::term::events::colors::bg, clr);
+                        else if (deed == e2::form::prop::colors::fg.id) boss.SIGNAL(tier::anycast, app::term::events::colors::fg, clr);
+                    };
+                    boss.SUBMIT(tier::anycast, e2::form::upon::started, root)
+                    {
+                        boss.start();
+                    };
+                    boss.SUBMIT(tier::anycast, app::term::events::search::forward, gear)
+                    {
+                        boss.search(gear, feed::fwd);
+                    };
+                    boss.SUBMIT(tier::anycast, app::term::events::search::reverse, gear)
+                    {
+                        boss.search(gear, feed::rev);
+                    };
+                });
             return window;
         };
     }
