@@ -183,8 +183,10 @@ utility like ctags is used to locate the definitions.
 
 )";
 
-        auto build = [](text cwd, text arg)
+        auto build = [](text cwd, text arg, xml::settings& config)
         {
+            auto highlight_color = skin::color(tone::highlight);
+
             auto window = ui::cake::ctor();
             window->plugin<pro::focus>()
                   ->plugin<pro::track>()
@@ -206,18 +208,18 @@ utility like ctags is used to locate the definitions.
                   });
             auto object = window->attach(ui::fork::ctor(axis::Y))
                                 ->colors(whitelt, 0xA05f1a00);
-                auto menu = object->attach(slot::_1, app::shared::main_menu());
+                auto menu = object->attach(slot::_1, app::shared::main_menu(config));
                 auto body_area = object->attach(slot::_2, ui::fork::ctor(axis::Y));
                     auto fields = body_area->attach(slot::_1, ui::pads::ctor(dent{ 1,1 }));
                         auto layers = fields->attach(ui::cake::ctor());
                             auto scroll = layers->attach(ui::rail::ctor())
                                                 ->plugin<pro::limit>(twod{ 4,3 }, twod{ -1,-1 });
                                 auto edit_box = scroll->attach(ui::post::ctor(true))
-                                                      ->plugin<pro::caret>(true, twod{ 25,1 }, faux)
+                                                      ->plugin<pro::caret>(true, faux, twod{ 25,1 })
                                                       ->colors(blackdk, whitelt)
                                                       ->upload(ansi::wrp(wrap::off).mgl(1)
                                                       .add(topic3)
-                                                      .fgc(app::shared::highlight_color)
+                                                      .fgc(highlight_color.bgc())
                                                       .add("From Wikipedia, the free encyclopedia"));
                     auto status_line = body_area->attach(slot::_2, ui::post::ctor())
                                                 ->plugin<pro::limit>(twod{ 1,1 }, twod{ -1,1 })
